import 'package:flutter/material.dart';
import 'package:awesome_notifications/awesome_notifications.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'dart:io';

String host = 'https://app.nativenotify.com';

class NativeNotify {
  static void initialize(flutterAppId, flutterAppToken) async {
    final prefs = await SharedPreferences.getInstance();

    await prefs.setInt('flutterAppId', flutterAppId);
    await prefs.setString('flutterAppToken', flutterAppToken);

    final flutterDeviceId = prefs.getInt('flutterDeviceId');

    if (flutterDeviceId == null) {
      try {
        var response = await http.post(Uri.parse('$host/api/flutter/device'),
            headers: {
              HttpHeaders.contentTypeHeader: 'application/json',
            },
            body: jsonEncode({
              'flutterAppId': '$flutterAppId',
              'flutterAppToken': flutterAppToken
            }));

        var jsonData = jsonDecode(response.body);

        await jsonData.forEach((key, value) async => {
              if (key == 'flutter_device_id')
                {await prefs.setInt('flutterDeviceId', value)}
            });

        print(
            "You can send Native Notify push notifications now. You're registered!");

        getPushNotification();
      } catch (e) {
        // print(e);
      }
    } else {
      getPushNotification();
    }

    AwesomeNotifications().initialize(null, [
      NotificationChannel(
          channelKey: 'mass',
          channelName: 'Native Notify Mass Notifications',
          channelDescription: "Notifications for all users",
          defaultColor: const Color(0XFF9050DD),
          ledColor: Colors.white,
          playSound: true,
          enableLights: true,
          enableVibration: true,
          importance: NotificationImportance.High)
    ]);

    AwesomeNotifications().isNotificationAllowed().then((isAllowed) {
      if (!isAllowed) {
        AwesomeNotifications().requestPermissionToSendNotifications();
      }
    });
  }
}

void checkForNotifications() async {
  final prefs = await SharedPreferences.getInstance();

  final flutterDeviceId = prefs.getInt('flutterDeviceId');
  final flutterAppId = prefs.getInt('flutterAppId');
  final flutterAppToken = prefs.getString('flutterAppToken');

  try {
    var response = await http.read(Uri.parse(
        '$host/api/flutter/notification/$flutterDeviceId/$flutterAppId/$flutterAppToken'));

    List<dynamic> notifications = jsonDecode(response);

    for (var i = 0; i < notifications.length; i++) {
      AwesomeNotifications().createNotification(
        content: NotificationContent(
            id: notifications[i]['flutter_notification_id'],
            channelKey: 'mass',
            title: notifications[i]['notification_title'],
            body: notifications[i]['notification_body']),
      );
    }
  } catch (e) {
    // print(e);
  }
}

void getPushNotification() {
  Future.delayed(const Duration(seconds: 15), () {
    return checkForNotifications();
  }).whenComplete(() => getPushNotification());
}
